@@ -1,4 +1,3 @@
-
 #ifndef DATA_INCLUDE
 #define DATA_INCLUDE
 /*------------------------------------------------------
@@ -20,27 +19,16 @@
                                                                                                                
  Peter Beerli 1996, Seattle
  beerli@genetics.washington.edu
- $Id: data.h,v 1.4 2000/05/03 00:35:30 beerli Exp $
+ $Id: data.h,v 1.7 2000/10/13 22:56:41 beerli Exp $
 -------------------------------------------------------*/
 
 #include "migration.h"
 
 extern void create_data (data_fmt ** data);
 extern void get_data (FILE * infile, data_fmt * data, option_fmt * options);
-extern void print_data (world_fmt * world, data_fmt * data,
-			option_fmt * options);
-extern void print_data_summary (FILE * file, world_fmt * world);
+extern void print_data (world_fmt * world, option_fmt * options, data_fmt * data);
+extern void print_data_summary (FILE * file, world_fmt * world, option_fmt *options, data_fmt *data);
 extern void free_datapart (data_fmt * data, option_fmt * options, long locus);
-extern void read_distance_fromfile (FILE * dfile, long tips, long nmlength,
-				    double **m);
-extern void read_geofile(data_fmt *data, option_fmt * options, long numpop);
+extern void read_distance_fromfile (FILE * dfile, long tips, long nmlength, double **m);
+extern void read_geofile (data_fmt * data, option_fmt * options, long numpop);
 #endif /*DATA_INCLUDE */
-
-
-
-
-
-
-
-
-
