@@ -14,12 +14,12 @@
   With help of Joe Felsenstein (joe@genetics.washington.edu),
   Mary Kuhner and Jon Yamato (mkkuhner@genetics.washington.edu)
 
-  $Id: definitions.h,v 1.37 2000/07/26 23:03:20 beerli Exp $
+  $Id: definitions.h,v 1.52 2001/04/08 04:24:44 beerli Exp $
 
   *----------------------------------------------------------------
   */
 #ifndef MIGRATEVERSION
-#define MIGRATEVERSION "0.9.6"
+#define MIGRATEVERSION "0.9.14"
 #endif
 
 #define MAINTAINER "Peter Beerli <beerli@genetics.washington.edu>"
@@ -35,7 +35,7 @@
 #endif
 
 #ifdef __MWERKS__
-#undef HAVE_LGAMMA 
+#undef HAVE_LGAMMA
 #define HAVE_STRFTIME 1
 #define HIGHBITS
 #define LAMARC_MALLOC
@@ -43,18 +43,16 @@
 #define MAC
 #include <ansi_prefix.mac.h>	/*fixes time problems */
 #include "unix.mac.h"
-#else  /*macintosh*/
+#else /*macintosh */
 #define DOS
 #undef HAVE_LGAMMA
-#endif /*windows*/
+#endif /*windows */
 #endif /*MWERKS*/
-
 /* we have found no strftime() and replace the time
    output with blanks*/
 #ifndef HAVE_STRFTIME
 #define NOTIME_FUNC
 #endif
-
 #define INFILE "infile"
 #define WEIGHTFILE "weightfile"
 #define CATFILE "catfile"
@@ -72,7 +70,7 @@
 #define DISTFILE "distfile"
 #define GEOFILE "geofile"
 #define BOOTFILE "bootstrapfile"
-
+#define UEPFILE "uepfile"
 #ifdef __STDIO_LOADED
 #define VMS
 #define MACHINE_TYPE "VAX/VMS C"
@@ -107,19 +105,16 @@ void vax_tweak_fmt (char *);
 #undef DBL_MAX
 #include <float.h>
 #endif /*__MWERKS__*/
-
 #ifdef HAVE_MALLOCWRAP
 /*if we use this then we guard against allocation problems */
 /* redefinitions of calloc, malloc, realloc are in sighandler.h*/
-#define LAMARC_MALLOC		
+#define LAMARC_MALLOC
 #endif
-
 #ifdef HAVE_LGAMMA
 #define LGAMMA lgamma
 #else
 #define LGAMMA mylgamma
 #endif
-
 #include "sighandler.h"
 #define FClose(file) if (file) fclose(file) ; file=NULL
 typedef unsigned char boolean;
@@ -137,15 +132,10 @@ typedef unsigned char boolean;
 #define FIRST 0
 /* the first population is the panel for snps*/
 #define PANEL 0
-#define SEQUENCETYPES "snu"
+#define SEQUENCETYPES "snuf"  /*s=sequence,n=linked snp, u=unlinked snp, f=ancestral reconstruction*/
 #define SNPTYPES      "nu"
-#define ALLELETYPES   "amb"
-/*
-#ifdef MAC
-MALLOCRETURN *mymalloc (long);
-#else
-MALLOCRETURN *mymalloc ();
-#endif *MAC*/
+#define ALLELETYPES   "amb" /*a=infinite allele,m=stepwise, b=brownian*/
+
 
 /*=================================================================
   SPECIFIC FOR MIGRATE
@@ -159,6 +149,7 @@ MALLOCRETURN *mymalloc ();
    MUST really be bigger than the number of 
    observed alleles, much bigger does not harm	*/
 #define MAXALLELES            93
+#define MAXBUFSIZE        100000      
 #define LINESIZE            1024	/* setting this smaller can break */
 #define STRSIZE              255	/* setting this smaller can break */
 #define DEFAULT_NMLENGTH      10	/* length of individual names */
@@ -171,9 +162,9 @@ MALLOCRETURN *mymalloc ();
 /* numerical borders and epsilons*/
 #define SMALL_VALUE       10e-21
 #define EPSILON         0.000001	/* a small number */
-#define SMALLEPSILON       1e-10    	/* a smaller number */
+#define SMALLEPSILON       1e-15 	/* a smaller number */
 #define BIGEPSILON         0.001	/* a not so small number */
-#define LONGCHAINEPSILON    1000	/* an unreasonable big number, 
+#define LONGCHAINEPSILON  10e100	/* an unreasonable big number, 
 					   so that the given number of 
 					   long chains is used */
 #define GELMAN_MYSTIC_VALUE 1.2
@@ -190,6 +181,7 @@ MALLOCRETURN *mymalloc ();
 #endif
 /* some math constants */
 #define LOGDBL_EPSILON -36.04365338911715608	/*N[Log[DBL_EPSILON] ,30] */
+#define LOG1 0.
 #define LOG2 0.693147180559945309417232121458	/*N[Log[2] ,30] */
 #define LOG2PIHALF -0.918938533204672741780329736406	/*N[Log[1/Sqrt[2 Pi]] ,30] */
 #define TWOPI 6.28318530717958647692528676656	/*N[Log[2 Pi] ,30] */
@@ -209,7 +201,7 @@ MALLOCRETURN *mymalloc ();
   these replace all the atoi atol etc and use the strto[lif] function*/
 #define ATOL(a) strtol((a),(char **) NULL, 10)
 #define ATOI(a) strtol((a),(char **) NULL, 10)
-#define ATOF(a) strtof((a),(char **) NULL, 10)
+#define ATOF(a) strtod((a),(char **) NULL, 10)
 /* theta (4 Ne mu) related material */
 #define SMALLEST_THETA	   1e-10
 #define BIGGEST_THETA	     1e4
@@ -224,7 +216,7 @@ MALLOCRETURN *mymalloc ();
 #define FRACTION_ALONG      0.66
 #define TIMELIST_GUESS       100
 #define SAMPLETREE_GUESS       1
-#define START_ALPHA          1.0
+#define START_ALPHA          10.0
 /* random material */
 #define AUTO                   0	/*+ use time() for seed */
 #define NOAUTO                 1	/*+ seed in parmfile   */
@@ -235,7 +227,7 @@ MALLOCRETURN *mymalloc ();
 #define EWENS                  2 /*- start values are Ewens estimate*/
 #define FST                    3	/*+ start values are FST estimate */
 #define RANDOMESTIMATE         4	/*+ start values are RANDOM estimate */
-#define PARAMGRID              5 /*+ use a range of values + replicate=yes:# */
+#define PARAMGRID              5	/*+ use a range of values + replicate=yes:# */
 /* defines for the migration0 values */
 /* define OWN                  0   + start values in parmfile */
 /* define FST                  3   + start values are FST estimates*/
@@ -250,7 +242,7 @@ MALLOCRETURN *mymalloc ();
 /* Maximizer and gamma things */
 #define NTRIALS             1000
 #define LOCI_NORM          0.00001
-#define GAMMA_INTERVALS      100
+#define GAMMA_INTERVALS       10
 /* definitions of migration model */
 #define MATRIX                 1	/*+ */
 #define MATRIX_SYMMETRIC      11	/*+ */
@@ -262,21 +254,22 @@ MALLOCRETURN *mymalloc ();
 #define CONTINUUM              4 /*- */
 #define NEIGHBOR               5 /*- */
 /* replication stuff */
-#define SINGLECHAIN            0        /*+ */
-#define MULTIPLECHAIN          1        /*+ */
-#define MULTIPLERUN            2        /*+ */
+#define SINGLECHAIN            0 /*+ */
+#define MULTIPLECHAIN          1 /*+ */
+#define MULTIPLERUN            2 /*+ */
 /* heating stuff */
 #define HEATED_CHAIN_NUM       4
 #define HEATED_THREAD_NUM      4
-#define COLD                   1         /*+ */
-#define WARM                   4         /*+ */
-#define HOT                    7         /*+ */
-#define VERYHOT                10        /*+ */
+#define COLD                   1	/*+ */
+#define WARM                   4	/*+ */
+#define HOT                    7	/*+ */
+#define VERYHOT                10	/*+ */
 /* maximization material*/
 #define SINGLELOCUS            0	/*+ */
 #define MULTILOCUS             1	/*+ */
 #define PROFILE                2	/*+ */
 /* print and plot material*/
+#define MAXPRINTVALLENGTH 16        /*+ maximal number of printpos*/
 #define PLOTALL                0	/*+ plot to outfile and mathfile */
 #define PLOTOUTFILE            1	/*+ plot only to outifle */
 #define PLOT4NM                0	/*+ use 4Nm instead of M=m/mu */
@@ -305,10 +298,10 @@ MALLOCRETURN *mymalloc ();
 #define SUMMARY                3	/*+ */
 #define MAX_PROFILE_TRIALS   100	/*+ */
 /* likelihood ratio stuff*/
-#define MEAN 0
-#define LOCUS 1
-#define MAXPOP   10		/*n*(n-1)+n) */
-#define MAXPARAM 100
+#define MLE 0
+#define ARBITRARY 1
+#define MAXPOP   100		/*n*(n-1)+n) */
+#define MAXPARAM 10000
 #define HEADER 1
 /* migration histogram material*/
 #define MIGHIST_ELEM 5
@@ -326,8 +319,3 @@ MALLOCRETURN *mymalloc ();
 #include <memdebug.h>
 #endif
 #endif /*definitions */
-
-
-
-
-
