@@ -13,9 +13,9 @@
 
  Peter Beerli 1996, Seattle
  beerli@genetics.washington.edu
- $Id: fst.h,v 1.1.1.1 1998/07/01 21:05:54 beerli Exp $
+ $Id: fst.h,v 1.2 2000/10/13 22:56:41 beerli Exp $
 -------------------------------------------------------*/
 extern void fst_type (char type);
-extern void calc_fst (world_fmt * world);
+extern void calc_fst (world_fmt * world, data_fmt *data);
 
 #endif /* FST_INCLUDE */
