@@ -10,12 +10,15 @@
 
  Peter Beerli 1996, Seattle
  beerli@genetics.washington.edu
- $Id: mcmc1.c,v 1.34 2000/07/26 23:03:20 beerli Exp $
+ $Id: mcmc1.c,v 1.52 2000/12/12 18:27:38 beerli Exp $
 -------------------------------------------------------*/
 #include "migration.h"
 #include "random.h"
 #include "tree.h"
 #include "mcmc2.h"
+#ifdef UEP
+#include "uep.h"
+#endif
 
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
@@ -30,60 +33,57 @@
 long metropolize (world_fmt * world, long g);
 /* private functions */
 void new_localtimelist (timelist_fmt ** ntl, timelist_fmt * otl, long numpop);
-void new_proposal (proposal_fmt ** proposal, timelist_fmt * tl,
-		   world_fmt * world);
+void new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world);
 void chooseOrigin (proposal_fmt * proposal);
-void construct_localtimelist (timelist_fmt * timevector,
-			      proposal_fmt * proposal);
-void traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
-		       long *oldnode_elem, int include_migration);
+void construct_localtimelist (timelist_fmt * timevector, proposal_fmt * proposal);
+void traverseAllNodes (node * theNode, node *** nodelist, long *node_elem, long *oldnode_elem, int include_migration);
 void add_locallineages (timelist_fmt * timevector, proposal_fmt * proposal);
-void chooseTarget (proposal_fmt * proposal, timelist_fmt * timevector,
-		   node ** bordernodes, long *bordernum);
-void findbordernodes (node * theNode, proposal_fmt * proposal, long pop,
-		      node ** bordernodes, long *bordernum,
-		      vtlist ** tyme, long gte);
+void chooseTarget (proposal_fmt * proposal, timelist_fmt * timevector, node ** bordernodes, long *bordernum);
+void findbordernodes (node * theNode, proposal_fmt * proposal, long pop, node ** bordernodes, long *bordernum, vtlist ** tyme, long gte);
 void free_proposal (proposal_fmt * proposal);
 void free_timevector (timelist_fmt * timevector);
 long remove_doublettes (timelist_fmt * timevector, node ** ptr);
 long xor (node ** ptrl1, node ** ptrl2);
 long rmigrcount (proposal_fmt * proposal);
 
-int migrate_old (proposal_fmt * proposal, node * up,
-		 long *old_migr_table_counter, boolean air);
+int migrate_old (proposal_fmt * proposal, node * up, long *old_migr_table_counter, boolean air);
 int migrate (proposal_fmt * proposal, node * up);
 int migrateb (proposal_fmt * proposal, node * up);
 
-int pre_population (proposal_fmt * proposal, vtlist * ltime, long gte,
-		    long *slider);
+int pre_population (proposal_fmt * proposal, vtlist * ltime, long gte, long *slider);
 
-boolean acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
-		    timelist_fmt * tyme);
-double eventtime (proposal_fmt * proposal, long pop, vtlist * tentry,
-		  char *event);
+boolean acceptlike (world_fmt * world, proposal_fmt * proposal, long g, timelist_fmt * tyme);
+double eventtime (proposal_fmt * proposal, long pop, vtlist * tentry, char *event);
 node *showsister (node * theNode);
 void count_migrations (node * p, long *count);
 long migration_from (long to, proposal_fmt * proposal);
 
 double prob_tree (world_fmt * world, timelist_fmt * tyme);
 
-void set_tree_dirty(node *p)
+/* Functions ++++++++++++++++++++++++++++++++++++++++++++++++*/
+void
+set_tree_dirty (node * p)
 {
-  switch(p->type)
+  switch (p->type)
     {
-    case 'm': set_dirty(p);set_tree_dirty(p->next->back);
+    case 'm':
+      set_dirty (p);
+      set_tree_dirty (p->next->back);
       break;
-    case 't': 
+    case 't':
       break;
     case 'i':
-      set_dirty(p);set_tree_dirty(p->next->back);
-      set_tree_dirty(p->next->next->back);
+      set_dirty (p);
+      set_tree_dirty (p->next->back);
+      set_tree_dirty (p->next->next->back);
       break;
     case 'r':
-      set_dirty(p);set_tree_dirty(p->next->back);
+      set_dirty (p);
+      set_tree_dirty (p->next->back);
       break;
     }
 }
+
 /*=======================================================*/
 long
 metropolize (world_fmt * world, long g)
@@ -96,12 +96,14 @@ metropolize (world_fmt * world, long g)
   long bordernum;
   long actualpop = -99, zz;
   double endtime, nexttime, age;
-
+#ifdef UEP
+  boolean uepsuccess = FALSE;
+#endif
   proposal_fmt *proposal;	/*scratchpad  in which all involved nodes
 				   are recorded and help arrays, e.g. migration arrays,
 				   are stored */
-  timelist_fmt *timevector;	/* local timelist */
-  vtlist *tentry;		/*pointer into timeslice */
+  timelist_fmt *timevector=NULL;	/* local timelist */
+  vtlist *tentry=NULL;		/*pointer into timeslice */
 
   /* ---------------------------------------------------------
      initialize local timelist and construct residual timelist 
@@ -140,9 +142,7 @@ metropolize (world_fmt * world, long g)
   slider = 0;
   while (nexttime <= endtime)
     {
-      actualpop = (proposal->migr_table_counter > 0) ?
-	proposal->migr_table[proposal->migr_table_counter - 1].from :
-	proposal->origin->pop;
+      actualpop = (proposal->migr_table_counter > 0) ? proposal->migr_table[proposal->migr_table_counter - 1].from : proposal->origin->pop;
       proposal->time = age + eventtime (proposal, actualpop, tentry, &event);
       if (proposal->time < nexttime)
 	{
@@ -159,9 +159,11 @@ metropolize (world_fmt * world, long g)
 	    }
 	  else
 	    {			/*coalesce */
-	      chooseTarget (proposal, timevector, proposal->bordernodes,
-			    &bordernum);
+	      chooseTarget (proposal, timevector, proposal->bordernodes, &bordernum);
 	      pretendcoalesce1p (proposal);
+#ifdef UEP
+	      uepsuccess = is_success_pseudo_uep (proposal);
+#endif
 	      coalesced = TRUE;
 	      break;
 	    }
@@ -172,39 +174,49 @@ metropolize (world_fmt * world, long g)
     }
   if (!coalesced)
     {
-      if (!pre_population
-	  (proposal, (*timevector).tl, (*timevector).T - 1, &slider))
+      if (!pre_population (proposal, (*timevector).tl, (*timevector).T - 1, &slider))
 	{
 	  free_proposal (proposal);
 	  free_timevector (timevector);
 	  return 0;
 	}
       pretendcoalesce1p (proposal);
+#ifdef UEP
+      uepsuccess = is_success_pseudo_uep (proposal);
+#endif
+
     }
-  if (acceptlike (world, proposal, g, timevector))
+  if (
+#ifdef UEP
+       ((!world->options->uep && !uepsuccess) || (world->options->uep && uepsuccess)) &&
+#endif
+       (acceptlike (world, proposal, g, timevector)))
     {
       if (proposal->time > world->root->tyme)
 	{			/*saveguard */
 	  world->root->tyme += proposal->time;
-//        fprintf (stdout, "ROOT moved from %f to %f because population joining moves to %f\n",
-//                 world->root->tyme - proposal->time, world->root->tyme, proposal->time);
-	}
+	}							       
       coalesce1p (proposal);
-      world->likelihood[g] = treelikelihood (world);	/*recalculate the p->x */
-      if (fabs (world->likelihood[g] - proposal->likelihood) > EPSILON)
+#ifdef UEP
+      world->likelihood[g] = treelikelihood (world);
+      if(world->options->uep)
 	{
-	  warning ("proposed and new likelihood differ: %f != %f\n",
-		   proposal->likelihood, world->likelihood[g]);
-	  warning ("abort the program and try to find the errors\n");
-	  warning ("there could be a wrong datatype, or infile\n");
-	  warning ("to check the data you can print it (see menu)\n");
+	  update_uep (world->root->next->back, world);
+	  check_uep_root(world->root->next->back, world);
+	  world->treelen = 0.0;
+	  calc_treelength(world->root->next->back,&world->treelen);
+	  world->ueplikelihood = ueplikelihood (world);
+	  world->likelihood[g] = world->ueplikelihood + world->likelihood[g];
 	}
+#else
+      world->likelihood[g] = treelikelihood (world);
+#endif
       /* create a new timelist */
       construct_tymelist (world, &world->treetimes[0]);
       if (world->options->treeprint != NONE)
 	print_tree (world, g, &treefilepos);
       world->migration_counts = 0;
-/* report the number of migration on the tree */
+      /* report the number of migration on the tree */
       count_migrations (world->root->next->back, &world->migration_counts);
       free_proposal (proposal);
       free_timevector (timevector);
@@ -223,15 +235,15 @@ void
 new_localtimelist (timelist_fmt ** ntl, timelist_fmt * otl, long numpop)
 {
   long i;
-  (*ntl) = (timelist_fmt *) calloc (1, sizeof (timelist_fmt) * 1);
-  (*ntl)[0].tl = (vtlist *) malloc ((*otl).allocT * sizeof (vtlist));
-  (*ntl)[0].allocT = (*otl).allocT;
-  (*ntl)[0].T = (*otl).T;
-  memcpy ((*ntl)[0].tl, (*otl).tl, (*otl).allocT * sizeof (vtlist));
-  for (i = 0; i < (*ntl)[0].allocT; i++)
+  (*ntl) = (timelist_fmt *) calloc (1, sizeof (timelist_fmt));
+  (*ntl)->tl = (vtlist *) malloc ((*otl).allocT * sizeof (vtlist));
+  (*ntl)->allocT = otl->allocT;
+  (*ntl)->T = otl->T;
+  memcpy ((*ntl)->tl, otl->tl, otl->allocT * sizeof (vtlist));
+  for (i = 0; i < (*ntl)->allocT; i++)
     {
-      (*ntl)[0].tl[i].lineages = (long *) calloc (1, sizeof (long) * numpop);
-      memcpy ((*ntl)[0].tl[i].lineages, (*otl).tl[i].lineages,
+      (*ntl)->tl[i].lineages = (long *) calloc (numpop, sizeof (long));
+      memcpy ((*ntl)->tl[i].lineages, otl->tl[i].lineages, 
 	      numpop * sizeof (long));
     }
 }
@@ -244,10 +256,8 @@ new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world)
   (*proposal) = (proposal_fmt *) calloc (1, sizeof (proposal_fmt));
   (*proposal)->listsize = ((*tl).allocT + (*tl).T + 5);
 
-  (*proposal)->aboveorigin =
-    (node **) calloc (1, sizeof (node *) * (*proposal)->listsize);
-  (*proposal)->bordernodes =
-    (node **) calloc (1, sizeof (node *) * (*proposal)->listsize);
+  (*proposal)->aboveorigin = (node **) calloc (1, sizeof (node *) * (*proposal)->listsize);
+  (*proposal)->bordernodes = (node **) calloc (1, sizeof (node *) * (*proposal)->listsize);
   (*proposal)->world = world;
   (*proposal)->datatype = world->options->datatype;
   (*proposal)->sumtips = world->sumtips;
@@ -259,20 +269,16 @@ new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world)
   (*proposal)->migration_model = world->options->migration_model;
   (*proposal)->line_f = (node **) calloc (1, sizeof (node *) * (*proposal)->sumtips);
   (*proposal)->line_t = (node **) calloc (1, sizeof (node *) * (*proposal)->sumtips);
+  (*proposal)->mf = (double *) calloc((*proposal)->endsite,sizeof(double));
+  (*proposal)->mt = (double *) calloc((*proposal)->endsite,sizeof(double));
   if (strchr (SEQUENCETYPES, (*proposal)->datatype))
     {
-
-      (*proposal)->xf.s =
-	(phenotype) malloc (world->data->seq->endsite * sizeof (ratelike *));
-      (*proposal)->xt.s =
-	(phenotype) malloc (world->data->seq->endsite * sizeof (ratelike *));
+      (*proposal)->xf.s = (phenotype) malloc (world->data->seq->endsite * sizeof (ratelike *));
+      (*proposal)->xt.s = (phenotype) malloc (world->data->seq->endsite * sizeof (ratelike *));
       for (j = 0; j < (*proposal)->endsite; j++)
 	{
-
-	  (*proposal)->xf.s[j] =
-	    (ratelike) malloc (world->options->rcategs * sizeof (sitelike));
-	  (*proposal)->xt.s[j] =
-	    (ratelike) malloc (world->options->rcategs * sizeof (sitelike));
+	  (*proposal)->xf.s[j] = (ratelike) malloc (world->options->rcategs * sizeof (sitelike));
+	  (*proposal)->xt.s[j] = (ratelike) malloc (world->options->rcategs * sizeof (sitelike));
 	}
     }
   else
@@ -282,22 +288,22 @@ new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world)
     }
   (*proposal)->old_migr_table_counter = 4 * (*proposal)->sumtips /* 100 */ ;
   (*proposal)->old_migr_table_counter2 = 4 * (*proposal)->sumtips /* 100 */ ;
-  (*proposal)->migr_table = (migr_table_fmt *) calloc (1,
-						       sizeof (migr_table_fmt)
-						       *
-						       (*proposal)->
-						       old_migr_table_counter);
-  (*proposal)->migr_table2 =
-    (migr_table_fmt *) calloc (1,
-			       sizeof (migr_table_fmt) *
-			       (*proposal)->old_migr_table_counter2);
+  (*proposal)->migr_table = (migr_table_fmt *) calloc (1, sizeof (migr_table_fmt) * (*proposal)->old_migr_table_counter);
+  (*proposal)->migr_table2 = (migr_table_fmt *) calloc ((*proposal)->old_migr_table_counter2, sizeof (migr_table_fmt));
   (*proposal)->migr_table_counter = 0;
   (*proposal)->migr_table_counter2 = 0;
-// precalculated values
+  // precalculated values
   (*proposal)->mig0list = world->mig0list;
   (*proposal)->design0list = world->design0list;
-
-
+#ifdef UEP
+  if (world->options->uep)
+    {
+      (*proposal)->ueplike = (double **) calloc (world->data->uepsites, sizeof (double *));
+      (*proposal)->ueplike[0] = (double *) calloc (world->numpop * world->data->uepsites, sizeof (double));
+      for (j = 1; j < world->data->uepsites; ++j)
+	(*proposal)->ueplike[j] = (*proposal)->ueplike[0] + j * world->numpop;
+    }
+#endif
 }
 
 
@@ -325,22 +331,9 @@ chooseOrigin (proposal_fmt * proposal)
 {
   long elem = 0, oldelem = (proposal->sumtips * 2);
   node *tmp, **goal;
-//  long rr;
-//  long i;
-
   goal = (node **) calloc (1, sizeof (node *) * oldelem);
-
-
-  traverseAllNodes (crawlback (proposal->root->next), &goal, &elem, &oldelem,
-		    NO_MIGR_NODES);
-  //jumblenodes (goal, elem - 2);
+  traverseAllNodes (crawlback (proposal->root->next), &goal, &elem, &oldelem, NO_MIGR_NODES);
   tmp = goal[RANDINT (0, elem - 2)];
-  // printf("o%li: %li\n",proposal->world->locus, tmp->actualpop);
-  // printf("o: %li ",rr);
-  //for(i=0;i<elem;i++)
-  //printf("%li ", goal[i]->actualpop);
-  //printf("\n");
-
   proposal->origin = tmp;
   if (proposal->origin != showtop (crawlback (proposal->root->next)))
     {
@@ -364,14 +357,10 @@ void
 construct_localtimelist (timelist_fmt * timevector, proposal_fmt * proposal)
 {
   long z = 0;
-/*     size_t num = 0; */
   long oz = proposal->listsize;
-//  proposal->mig_removed = FALSE;
   timevector->numpop = proposal->numpop;
-  traverseAllNodes (crawlback (proposal->origin)->back,
-		    &proposal->aboveorigin, &z, &oz, WITH_MIGR_NODES);
+  traverseAllNodes (crawlback (proposal->origin)->back, &proposal->aboveorigin, &z, &oz, WITH_MIGR_NODES);
   proposal->aboveorigin[z++] = proposal->oback;
-  /*    num = (size_t) z; */
   z = remove_doublettes (timevector, proposal->aboveorigin);
   qsort ((void *) (*timevector).tl, (*timevector).T, sizeof (vtlist), agecmp);
   (*timevector).T -= z;
@@ -392,8 +381,7 @@ the macros NO_MIGR_NODES=0 and WITH_MIGR_NODES=1 in the treesetup.h file
 PB 1995
  */
 void
-traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
-		  long *oldnode_elem, int include_migration)
+traverseAllNodes (node * theNode, node *** nodelist, long *node_elem, long *oldnode_elem, int include_migration)
 {
   long elem;
   if (include_migration == NO_MIGR_NODES)
@@ -401,17 +389,14 @@ traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
       if (theNode->type != 't')
 	{
 	  if (crawlback (theNode->next) != NULL)
-	    traverseAllNodes (crawlback (theNode->next),
-			      nodelist, node_elem, oldnode_elem, NO_MIGR_NODES);
+	    traverseAllNodes (crawlback (theNode->next), nodelist, node_elem, oldnode_elem, NO_MIGR_NODES);
 	  if (theNode->type != 'm' && crawlback (theNode->next->next) != NULL)
-	    traverseAllNodes (crawlback (theNode->next->next),
-			      nodelist, node_elem, oldnode_elem, NO_MIGR_NODES);
+	    traverseAllNodes (crawlback (theNode->next->next), nodelist, node_elem, oldnode_elem, NO_MIGR_NODES);
 	  if ((*node_elem) == (*oldnode_elem - 1))
 	    {
 	      elem = *oldnode_elem = ((*oldnode_elem) + (*oldnode_elem) / 2);
 	      (*nodelist) = (node **) realloc ((*nodelist), sizeof (node *) * elem);
-	      memset ((*nodelist) + (*oldnode_elem), 0,
-		      sizeof (node *) * (elem - (*oldnode_elem)));
+	      memset ((*nodelist) + (*oldnode_elem), 0, sizeof (node *) * (elem - (*oldnode_elem)));
 	      *oldnode_elem = elem;
 	    }
 	  (*nodelist)[(*node_elem)++] = theNode;
@@ -426,8 +411,7 @@ traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
 	    {
 	      elem = *oldnode_elem = ((*oldnode_elem) + (*oldnode_elem) / 2);
 	      (*nodelist) = (node **) realloc ((*nodelist), sizeof (node *) * elem);
-	      memset ((*nodelist) + (*oldnode_elem), 0,
-		      sizeof (node *) * (elem - (*oldnode_elem)));
+	      memset ((*nodelist) + (*oldnode_elem), 0, sizeof (node *) * (elem - (*oldnode_elem)));
 	      *oldnode_elem = elem;
 	    }
 	  (*nodelist)[(*node_elem)] = theNode;
@@ -439,19 +423,14 @@ traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
       if (theNode->type != 't')
 	{
 	  if (theNode->next->back != NULL)
-	    traverseAllNodes (theNode->next->back,
-			      nodelist, node_elem, oldnode_elem,
-			      WITH_MIGR_NODES);
+	    traverseAllNodes (theNode->next->back, nodelist, node_elem, oldnode_elem, WITH_MIGR_NODES);
 	  if (theNode->type != 'm' && theNode->next->next->back != NULL)
-	    traverseAllNodes (theNode->next->next->back,
-			      nodelist, node_elem, oldnode_elem,
-			      WITH_MIGR_NODES);
+	    traverseAllNodes (theNode->next->next->back, nodelist, node_elem, oldnode_elem, WITH_MIGR_NODES);
 	  if ((*node_elem) == (*oldnode_elem - 1))
 	    {
 	      elem = *oldnode_elem = ((*oldnode_elem) + (*oldnode_elem) / 2);
 	      (*nodelist) = (node **) realloc ((*nodelist), sizeof (node *) * elem);
-	      memset ((*nodelist) + (*oldnode_elem), 0,
-		      sizeof (node *) * (elem - (*oldnode_elem)));
+	      memset ((*nodelist) + (*oldnode_elem), 0, sizeof (node *) * (elem - (*oldnode_elem)));
 	      *oldnode_elem = elem;
 	    }
 	  (*nodelist)[(*node_elem)++] = theNode;
@@ -462,8 +441,7 @@ traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
 	    {
 	      elem = *oldnode_elem = ((*oldnode_elem) + (*oldnode_elem) / 2);
 	      (*nodelist) = (node **) realloc ((*nodelist), sizeof (node *) * elem);
-	      memset ((*nodelist) + (*oldnode_elem), 0,
-		      sizeof (node *) * (elem - (*oldnode_elem)));
+	      memset ((*nodelist) + (*oldnode_elem), 0, sizeof (node *) * (elem - (*oldnode_elem)));
 	      *oldnode_elem = elem;
 	    }
 	  (*nodelist)[(*node_elem)++] = theNode;
@@ -502,8 +480,7 @@ add_locallineages (timelist_fmt * timevector, proposal_fmt * proposal)
   if (timevector->T == 1)
     timevector->tl[0].lineages[proposal->osister->actualpop] += 1;
   else
-    timevector->tl[timevector->T -
-		   1].lineages[timevector->tl[timevector->T - 2].from] += 1;
+    timevector->tl[timevector->T - 1].lineages[timevector->tl[timevector->T - 2].from] += 1;
   add_partlineages (numpop, &timevector);
 
 }
@@ -536,9 +513,10 @@ xor (node ** ptrl1, node ** ptrl2)
 
    PROPOSAL proposal-scratchpad
    UP       node above (younger)
-   MIGR_TABLE_COUNTER migration array counter, will increase by one during execution
-   AIR      if true standard execution, if false updating the last lineage in the 
-   residual tree.
+   MIGR_TABLE_COUNTER migration array counter, 
+   will increase by one during execution
+   AIR      if true standard execution, if false updating the last 
+   lineage in the residual tree.
  */
 int
 migrate (proposal_fmt * proposal, node * up)
@@ -559,14 +537,12 @@ migrate (proposal_fmt * proposal, node * up)
   if (i >= proposal->old_migr_table_counter)
     {
       proposal->old_migr_table_counter += 10;
-      proposal->migr_table =
-	(migr_table_fmt *) realloc (proposal->migr_table,
-				    sizeof (migr_table_fmt) *
-				    (proposal->old_migr_table_counter));
+      proposal->migr_table = (migr_table_fmt *) realloc (proposal->migr_table, sizeof (migr_table_fmt) * (proposal->old_migr_table_counter));
     }
   proposal->migr_table_counter = i;
   return 1;
 }
+
 int
 migrateb (proposal_fmt * proposal, node * up)
 {
@@ -587,18 +563,14 @@ migrateb (proposal_fmt * proposal, node * up)
   if (i >= proposal->old_migr_table_counter2)
     {
       proposal->old_migr_table_counter2 += 10;
-      proposal->migr_table2 =
-	(migr_table_fmt *) realloc (proposal->migr_table2,
-				    sizeof (migr_table_fmt) *
-				    (proposal->old_migr_table_counter2));
+      proposal->migr_table2 = (migr_table_fmt *) realloc (proposal->migr_table2, sizeof (migr_table_fmt) * (proposal->old_migr_table_counter2));
     }
   proposal->migr_table_counter2 = i;
   return 1;
 }
 
 int
-migrate_old (proposal_fmt * proposal, node * up, long *old_migr_table_counter,
-	     boolean air)
+migrate_old (proposal_fmt * proposal, node * up, long *old_migr_table_counter, boolean air)
 {
   migr_table_fmt *array;
   long i;
@@ -645,18 +617,12 @@ migrate_old (proposal_fmt * proposal, node * up, long *old_migr_table_counter,
       (*old_migr_table_counter) += 10;
       if (air)
 	{
-	  proposal->migr_table =
-	    (migr_table_fmt *) realloc (proposal->migr_table,
-					sizeof (migr_table_fmt) *
-					(*old_migr_table_counter));
+	  proposal->migr_table = (migr_table_fmt *) realloc (proposal->migr_table, sizeof (migr_table_fmt) * (*old_migr_table_counter));
 	  array = proposal->migr_table;
 	}
       else
 	{
-	  proposal->migr_table2 =
-	    (migr_table_fmt *) realloc (proposal->migr_table2,
-					sizeof (migr_table_fmt) *
-					(*old_migr_table_counter));
+	  proposal->migr_table2 = (migr_table_fmt *) realloc (proposal->migr_table2, sizeof (migr_table_fmt) * (*old_migr_table_counter));
 	  array = proposal->migr_table;
 	}
     }
@@ -670,12 +636,13 @@ migrate_old (proposal_fmt * proposal, node * up, long *old_migr_table_counter,
     }
   return 1;
 }
+
 /* migration_from() returns the FROM population when there was a migration 
    TO        population to migrate to
    PROPOSAL  proposal-scratchpad
  */
 long
-migration_from (long to, proposal_fmt * proposal)
+migration_from_old (long to, proposal_fmt * proposal)
 {
   long j, ii, msta, msto;
   double *geo = proposal->world->data->geo;
@@ -700,28 +667,24 @@ migration_from (long to, proposal_fmt * proposal)
     return ++ii;
 }
 
-//  
-//  if (to <= ii)
-//    ii++;
-/* debug
-  switch(ii){
-      case 0: one++; break;
-      case 1: two++; break;
-      case 2: three++; break;
-      case 3: four++; break;
-      default:
-          error("illegal pop number in migration_from()");
-  }
-  debug[proposal->numpop * to + ii]+=1;
-//end debug
-*/
-//  return ii;
-//}
-
+long
+migration_from (long to, proposal_fmt * proposal)
+{
+  long ii=0;
+  double *r = proposal->world->migproblist[to];
+  double rr = RANDUM ();
+  while (rr > r[ii])
+    {
+      ii++;
+    }
+  if (ii < to)
+    return ii;
+  else
+    return ++ii;
+}
 
 void
-chooseTarget (proposal_fmt * proposal, timelist_fmt * timevector,
-	      node ** bordernodes, long *bordernum)
+chooseTarget (proposal_fmt * proposal, timelist_fmt * timevector, node ** bordernodes, long *bordernum)
 {
   long actualpop = -99;
   node *rb = crawlback (proposal->root->next);
@@ -736,8 +699,7 @@ chooseTarget (proposal_fmt * proposal, timelist_fmt * timevector,
     {
       error ("Wrong Time for action in chooseTarget()\n");
     }
-  findbordernodes (rb, proposal, actualpop, bordernodes, bordernum,
-		   &(*timevector).tl, (*timevector).T);
+  findbordernodes (rb, proposal, actualpop, bordernodes, bordernum, &(*timevector).tl, (*timevector).T);
   if (*bordernum > 0)
     {
       proposal->target = bordernodes[RANDINT (0, (*bordernum) - 1)];
@@ -762,9 +724,7 @@ chooseTarget (proposal_fmt * proposal, timelist_fmt * timevector,
 }
 
 void
-findbordernodes_old (node * theNode, proposal_fmt * proposal, long pop,
-		     node ** bordernodes, long *bordernum,
-		     vtlist ** tyme, long gte)
+findbordernodes (node * theNode, proposal_fmt * proposal, long pop, node ** bordernodes, long *bordernum, vtlist ** tyme, long gte)
 {
   node *tmp, *back;
   if (theNode == proposal->oback)
@@ -777,44 +737,7 @@ findbordernodes_old (node * theNode, proposal_fmt * proposal, long pop,
       tmp = showtop (theNode);
       back = showtop (theNode->back);
     }
-  if (pop == tmp->pop && pop == back->actualpop &&
-      tmp->tyme < proposal->time && back->tyme > proposal->time)
-    {
-      bordernodes[(*bordernum)++] = tmp;
-    }
-  else
-    {
-      if (back->tyme < proposal->time)
-	return;
-      if (tmp->type!='t')
-	{
-	  if (tmp->next->back != NULL)
-	    findbordernodes (tmp->next->back, proposal, pop, bordernodes,
-			     bordernum, tyme, gte);
-	  if (tmp->type != 'm' && tmp->next->next->back != NULL)
-	    findbordernodes (tmp->next->next->back, proposal, pop,
-			     bordernodes, bordernum, tyme, gte);
-	}
-    }
-}
-
-void
-findbordernodes (node * theNode, proposal_fmt * proposal, long pop,
-		 node ** bordernodes, long *bordernum, vtlist ** tyme, long gte)
-{
-  node *tmp, *back;
-  if (theNode == proposal->oback)
-    {
-      tmp = showtop (crawlback (proposal->osister)->back);
-      back = showtop (proposal->oback->back);
-    }
-  else
-    {
-      tmp = showtop (theNode);
-      back = showtop (theNode->back);
-    }
-  if (pop == tmp->pop && pop == back->actualpop &&
-      tmp->tyme < proposal->time && back->tyme > proposal->time)
+  if (pop == tmp->pop && pop == back->actualpop && tmp->tyme < proposal->time && back->tyme > proposal->time)
     {
       bordernodes[(*bordernum)++] = tmp;
       return;
@@ -823,14 +746,12 @@ findbordernodes (node * theNode, proposal_fmt * proposal, long pop,
     {
       if (back->tyme < proposal->time)
 	return;
-      if (tmp->type!='t')
+      if (tmp->type != 't')
 	{
 	  if (tmp->next->back != NULL)
-	    findbordernodes (tmp->next->back, proposal, pop, bordernodes,
-			     bordernum, tyme, gte);
+	    findbordernodes (tmp->next->back, proposal, pop, bordernodes, bordernum, tyme, gte);
 	  if (tmp->type != 'm' && tmp->next->next->back != NULL)
-	    findbordernodes (tmp->next->next->back, proposal, pop,
-			     bordernodes, bordernum, tyme, gte);
+	    findbordernodes (tmp->next->next->back, proposal, pop, bordernodes, bordernum, tyme, gte);
 	}
     }
 }
@@ -886,14 +807,8 @@ pre_population (proposal_fmt * proposal, vtlist * ltime, long gte, long *slider)
     }
   proposal->tsister = NULL;
   pop2 = proposal->realtarget->pop;
-  pop1 = proposal->migr_table_counter > 0 ?
-    proposal->migr_table[proposal->migr_table_counter -
-			 1].from : proposal->origin->pop;
-  age1 =
-    MAX (proposal->realtarget->tyme,
-	 proposal->migr_table_counter >
-	 0 ? proposal->migr_table[proposal->migr_table_counter -
-				  1].time : proposal->origin->tyme);
+  pop1 = proposal->migr_table_counter > 0 ? proposal->migr_table[proposal->migr_table_counter - 1].from : proposal->origin->pop;
+  age1 = MAX (proposal->realtarget->tyme, proposal->migr_table_counter > 0 ? proposal->migr_table[proposal->migr_table_counter - 1].time : proposal->origin->tyme);
   horizon = MAX (proposal->oback->tyme, age1);
   while (age1 < horizon)
     {
@@ -933,9 +848,7 @@ pre_population (proposal_fmt * proposal, vtlist * ltime, long gte, long *slider)
 	    {
 	      return 0;
 	    }
-	  pop1 = proposal->migr_table_counter > 0 ?
-	    proposal->migr_table[proposal->migr_table_counter - 1].from :
-	    proposal->origin->pop;
+	  pop1 = proposal->migr_table_counter > 0 ? proposal->migr_table[proposal->migr_table_counter - 1].from : proposal->origin->pop;
 	}
     }
   age1 = horizon;
@@ -1001,9 +914,7 @@ pre_population (proposal_fmt * proposal, vtlist * ltime, long gte, long *slider)
 	    {
 	      return 0;		/* migration limit reached */
 	    }
-	  pop1 = proposal->migr_table_counter > 0 ?
-	    proposal->migr_table[proposal->migr_table_counter - 1].from :
-	    proposal->origin->pop;
+	  pop1 = proposal->migr_table_counter > 0 ? proposal->migr_table[proposal->migr_table_counter - 1].from : proposal->origin->pop;
 	}
       else
 	{
@@ -1013,9 +924,7 @@ pre_population (proposal_fmt * proposal, vtlist * ltime, long gte, long *slider)
 	    {
 	      return 0;		/* migration limit reached */
 	    }
-	  pop2 = proposal->migr_table_counter2 > 0 ?
-	    proposal->migr_table2[proposal->migr_table_counter2 - 1].from :
-	    proposal->realtarget->pop;
+	  pop2 = proposal->migr_table_counter2 > 0 ? proposal->migr_table2[proposal->migr_table_counter2 - 1].from : proposal->realtarget->pop;
 	}
     }
   error ("Reached the end of function without coalescing");
@@ -1030,6 +939,8 @@ free_proposal (proposal_fmt * proposal)
   free (proposal->bordernodes);
   free (proposal->line_f);
   free (proposal->line_t);
+  free (proposal->mf);
+  free (proposal->mt);
   if (strchr (SEQUENCETYPES, proposal->datatype))
     {
       for (j = 0; j < proposal->endsite; j++)
@@ -1047,6 +958,13 @@ free_proposal (proposal_fmt * proposal)
     }
   free (proposal->migr_table);
   free (proposal->migr_table2);
+#ifdef UEP
+  if (proposal->world->options->uep)
+    {
+      free (proposal->ueplike[0]);
+      free (proposal->ueplike);
+    }
+#endif
   free (proposal);
 }
 
@@ -1054,44 +972,37 @@ void
 free_timevector (timelist_fmt * timevector)
 {
   long i;
-  for (i = 0; i < timevector[0].allocT; i++)
+  for (i = 0; i < timevector->allocT; i++)
     {
-      free (timevector[0].tl[i].lineages);
+      free (timevector->tl[i].lineages);
     }
-  free (timevector[0].tl);
+  free (timevector->tl);
   free (timevector);
 }
 
 /*----------------------------------------------------------*
  * rejection/acceptance of the new tree according to the likelihood
- * and an acceptance ratio which is higher the btter the
+ * and an acceptance ratio which is higher the better the
  * likelihood values are (-> Metropolis)
  */
 boolean
-acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
-	    timelist_fmt * tyme)
+acceptlike (world_fmt * world, proposal_fmt * proposal, long g, timelist_fmt * tyme)
 {
-  //static boolean done = FALSE, 
-  boolean report = TRUE; 
-  long oldg = -1;//, acceptprob=0;
-  //static 
+#ifdef UEP
+  node *first;
+#endif
+  boolean report = TRUE;
+  long oldg = -1;
   double *zz;
-  double rr, expo;//, Qnew, Qold;
-  long rm;//double check;
+  double rr, expo;
+  long rm;
   long rmc = rmigrcount (proposal);
-  //if (!done)
-  //  {
-  //    zz = world->heat;
-  //    done = TRUE;
-  //  }
   zz = world->heat;
-  if ((rm = proposal->migr_table_counter + proposal->migr_table_counter2 +
-       world->migration_counts - rmc) > MIGRATION_LIMIT * world->numpop)
+  if ((rm = proposal->migr_table_counter + proposal->migr_table_counter2 + world->migration_counts - rmc) > MIGRATION_LIMIT * world->numpop)
     {
       if (report || g != oldg)
 	{
-	  warning ("migration limit (%li) exceeded: %li\n",
-		   MIGRATION_LIMIT * world->numpop, rm);
+	  warning ("migration limit (%li) exceeded: %li\n", MIGRATION_LIMIT * world->numpop, rm);
 	  warning ("results may be underestimating migration rates\n");
 	  warning ("for this chain\n");
 	  report = FALSE;
@@ -1099,17 +1010,26 @@ acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
 	}
       return FALSE;
     }
-  //set_tree_dirty(world->root);
-  //check = treelikelihood (world);
-  //if(check!=world->likelihood[g])
-  //  error("treelike and stored treelike are not the same, out of sync!");
+#ifdef UEP
+  if (world->options->uep)
+    {
+      first = first_uep2 (proposal->world->root->next->back, proposal->world->root->next->back, proposal->world->data->uepsites);
+      proposal->firstuep = first_uep (first, NULL, proposal->world->data->uepsites);
+      proposal->ueplikelihood = pseudo_ueplikelihood (world, proposal);
+      proposal->likelihood = pseudotreelikelihood (world, proposal);
+      //      printf("%s> pl=%f pul=%f\n",proposal->world->name,proposal->likelihood,proposal->ueplikelihood);
+      proposal->likelihood += proposal->ueplikelihood;
+    }
+  else
+    {
+      proposal->likelihood = pseudotreelikelihood (world, proposal);
+    }
+#else
   proposal->likelihood = pseudotreelikelihood (world, proposal);
+#endif
   if (world->likelihood[g] < proposal->likelihood)
     {
-      //  fprintf(stdout,"%-10s:%10.5f proposal:%10.5f heat:%4.2f => accept\n",
-      //	      world->name,world->likelihood[g],proposal->likelihood,
-      //	      world->heat[0]);
-    return TRUE;
+      return TRUE;
     }
   if (!world->options->heating)
     {
@@ -1120,32 +1040,13 @@ acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
     }
   else
     {
-      //Qold = prob_tree (world, world->treetimes);
-      //Qnew = prob_tree (world, tyme);
-      expo = //(Qold - Qnew) * (-1. + 1./ zz[world->actualinc]) +
-	(proposal->likelihood - world->likelihood[g]) * world->heat[0]; 
-// * zz[world->actualinc];
-      // if(world->actualinc==0)
-      //	acceptprob=0;
-      //else
-      //	{
-      //	  acceptprob += expo;
-      //	  if (world->actualinc==world->increment-1)
-      //	    {
+      expo = (proposal->likelihood - world->likelihood[g]) * world->heat[0];
       rr = log (RANDUM ());
-      //fprintf(stdout,"%-10s:%10.5f proposal:%10.5f heat:%4.2f => %f<%f\n",
-      //	      world->name,world->likelihood[g],proposal->likelihood,
-      //	      world->heat[0], rr, expo);
-      //	      fprintf(stderr,"%s (%f-%f)**(-1+1/%f) => %f \n",rr<acceptprob ? "+" : "-",Qnew,Qold,zz[world->actualinc],expo); 
       if (rr < expo)
 	return TRUE;
-      // else
-      //	return FALSE;
-      //    }
-      //	  return TRUE;
-      //    }
-      // }
     }
+  //printf("debug: always true\n");
+  //return TRUE;
   return FALSE;
 }
 
@@ -1195,8 +1096,7 @@ eventtime (proposal_fmt * proposal, long pop, vtlist * tentry, char *event)
 		      if (mig_force)
 			{
 			  warning ("Migration forced\n");
-			  warning
-			    ("results may be overestimating migration rates\n");
+			  warning ("results may be overestimating migration rates\n");
 			  warning ("for this chain\n");
 			  mig_force = FALSE;
 			}
@@ -1271,7 +1171,7 @@ prob_tree (world_fmt * world, timelist_fmt * tyme)
   tyme->tl[0].interval = tyme->tl[0].age;
   for (j = 1; j < tyme->T; j++)
     {
-      tyme->tl[j].interval = tyme->tl[j].age -tyme->tl[j-1].age;
+      tyme->tl[j].interval = tyme->tl[j].age - tyme->tl[j - 1].age;
     }
   for (j = 0; j < tyme->T - 1; j++)
     {
@@ -1279,8 +1179,7 @@ prob_tree (world_fmt * world, timelist_fmt * tyme)
       for (pop = 0; pop < world->numpop; pop++)
 	{
 	  mm += tyme->tl[j].lineages[pop] * world->mig0list[pop];
-	  cc += tyme->tl[j].lineages[pop] * (tyme->tl[j].lineages[pop] - 1) /
-	    world->param0[pop];
+	  cc += tyme->tl[j].lineages[pop] * (tyme->tl[j].lineages[pop] - 1) / world->param0[pop];
 	}
       ss += -(tyme->tl[j].interval) * (mm + cc);
       if (tyme->tl[j].from == tyme->tl[j].to)
