@@ -13,57 +13,68 @@
                                                                                                                
  Peter Beerli 1996, Seattle
  beerli@genetics.washington.edu
- $Id: random.c,v 1.9 2000/07/26 23:03:20 beerli Exp $
+ $Id: random.c,v 1.22 2001/04/09 17:14:49 beerli Exp $
 -------------------------------------------------------*/
 
 #include "sighandler.h"
 #include "migration.h"
-
+#include "heating.h"
+#include "random.h"
+#include "migrate_mpi.h"
+#ifdef SPRNG /*NCSA random number generator library, slower than ours*/
+#define SIMPLE_SPRNG
+#ifdef MPI
+#ifndef USE_MPI
+#define USE_MPI
+#endif
+#endif
+#include "sprng.h"
+#endif
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
 #endif
-
-
-/*extern gettimeofday(); */
-/* prototypes ----------------------------------------- */
-void getseed (option_fmt * options);
-
-#ifdef HIGHBITS
-double mk_randum (void);
-#define RANDUM() mk_randum()
-#define RANDINT(a,b) (long)((a)+(mk_randum()* (((b)-(a))+1.)))
-#else
-double jf_randum (void);
-#define RANDUM() jf_randum()
-#define RANDINT(a,b) (long)((a)+(jf_randum()* (((b)-(a))+1.)))
+#ifdef PTHREADS
+extern tpool_t heating_pool;
 #endif
 
+/* prototypes ----------------------------------------- */
+void getseed (option_fmt * options);
+void swap_ptr(void * ptr, void * newptr);
 
+double randum (void);
+#ifdef PTHREADS
+double randum_thread (void);
+#endif
+
+//=============================================
 void
 getseed (option_fmt * options)
 {
+#ifndef SPRNG
   long i;
-  unsigned long timeseed;
+#endif
+  long timeseed;
 
   switch (options->autoseed)
     {
     case AUTO:
-      //#ifndef MAC
-      timeseed = (unsigned long) time (NULL);
-
-      //#else
-      //timeseed = (unsigned long) clock () / 4;
-      //#endif
-      switch(timeseed % 4)
+      timeseed = abs((long) time (NULL));
+      switch (timeseed % 4)
 	{
-	case 0: ++timeseed; break;
-	case 1: break;
-	case 2: ++timeseed; break;
-	case 3: break;
-	default: error("died in in automatic random number seed assignment");
+	case 0:
+	  ++timeseed;
+	  break;
+	case 1:
+	  break;
+	case 2:
+	  ++timeseed;
+	  break;
+	case 3:
+	  break;
+	default:
+	  error ("Problem with automatic random number seed assignment");
 	}
-      
-      options->inseed = (long) timeseed;
+      options->inseed = abs((long) timeseed);
       break;
     case NOAUTO:
       break;
@@ -73,10 +84,17 @@ getseed (option_fmt * options)
       error ("Error: Seed value not defined");
       break;
     }
+#ifdef MPI
+  options->inseed = abs(options->inseed + (myID * 100000.));
+  printf("%i> random number seed = %li\n", myID, options->inseed); 
+#endif
   options->saveseed = options->inseed;
-  for (i = 1; i <= 1000; i++)	/* clear the random numbergenerator */
+#ifdef SPRNG
+  init_sprng(options->inseed, SPRNG_DEFAULT);
+#else /* if we do not have SPRNG */
+  for (i = 1; i <= 1000; i++)	/* clear the random number generator */
     RANDUM ();
-#ifdef HIGHBITS
+
   for (i = 0; i <= 2; i++)
     seed[i] = 0;
   i = 0;
@@ -87,28 +105,31 @@ getseed (option_fmt * options)
       i++;
     }
   while (options->inseed != 0);
-#else
-  for (i = 0; i <= 5; i++)
-    seed[i] = 0;
-  i = 0;
-  do
-    {
-      seed[i] = options->inseed & 63;
-      options->inseed /= 64;
-      i++;
-    }
-  while (options->inseed != 0);
-#endif
+#endif /* not SPRNG*/
 }
 
-#ifdef HIGHBITS
-double
-mk_randum (void)
-/* Mary's version--faster but needs 32 bits.  Loops have been unrolled
-   for speed. */
+void   swap_ptr(void * ptr, void * newptr)
 {
-  longer newseed;
+  void * temp;
+  temp = ptr;
+  ptr = newptr;
+  newptr = temp;
+}
 
+#ifdef PTHREADS
+double
+randum_thread (void)
+/* thread save random number generator */
+{
+#ifdef SPRNG
+  return sprng();
+#else
+  double value;
+  longer newseed;
+#ifdef PTHREADS
+  if ((pthread_mutex_lock (&(heating_pool->random_lock))) != 0)
+    error ("pthread_mutex_lock failed in random_thread()");
+#endif
   newseed[0] = 1549 * seed[0];
   newseed[1] = newseed[0] / 2048;
   newseed[0] &= 2047;
@@ -121,44 +142,48 @@ mk_randum (void)
 
   memcpy (seed, newseed, sizeof (longer));
   seed[2] &= 1023;
-  return (((seed[0] / 2048.0 + seed[1]) / 2048.0 + seed[2]) / 1024.0);
+  value = (((seed[0] / 2048.0 + seed[1]) / 2048.0 + seed[2]) / 1024.0);
+#ifdef PTHREADS
+  if ((pthread_mutex_unlock (&(heating_pool->random_lock))) != 0)
+    error ("pthread_mutex_unlock failed in randum_thread()");
+#endif 
+  return value;
+#endif /*SPRNG*/
 }				/* randum */
 #else
 double
-jf_randum (void)
-{				/* randum -- slow but machine independent */
-  /* random number generator -- slow but machine independent */
-  long i, j, k, sum;
-  longer mult, newseed;
-  double x;
-
-  mult[0] = 13;
-  mult[1] = 24;
-  mult[2] = 22;
-  mult[3] = 6;
-  for (i = 0; i <= 5; i++)
-    newseed[i] = 0;
-  for (i = 0; i <= 5; i++)
-    {
-      sum = newseed[i];
-      k = i;
-      if (i > 3)
-	k = 3;
-      for (j = 0; j <= k; j++)
-	sum += mult[j] * seed[i - j];
-      newseed[i] = sum;
-      for (j = i; j <= 4; j++)
-	{
-	  newseed[j + 1] += newseed[j] / 64;
-	  newseed[j] &= 63;
-	}
-    }
+randum (void)
+/* Non thread-safe random number generator (faster) */
+{
+#ifdef SPRNG
+  return sprng();
+#else
+  double value;
+  longer newseed;
+  newseed[0] = 1549 * seed[0];
+  newseed[1] = newseed[0] / 2048;
+  newseed[0] &= 2047;
+  newseed[2] = newseed[1] / 2048;
+  newseed[1] &= 2047;
+  newseed[1] += 1549 * seed[1] + 812 * seed[0];
+  newseed[2] += newseed[1] / 2048;
+  newseed[1] &= 2047;
+  newseed[2] += 1549 * seed[2] + 812 * seed[1];
   memcpy (seed, newseed, sizeof (longer));
-  seed[5] &= 3;
-  x = 0.0;
-  for (i = 0; i <= 5; i++)
-    x = x / 64.0 + seed[i];
-  x /= 4.0;
-  return x;
+  //swap_ptr(&seed,&newseed);
+  seed[2] &= 1023;
+  value = (((seed[0] / 2048.0 + seed[1]) / 2048.0 + seed[2]) / 1024.0);
+  return value;
+#endif /*SPRNG*/
 }				/* randum */
 #endif
+
+
+
+
+
+
+
+
+
+
