@@ -11,7 +11,7 @@
                                                                                                                
  Peter Beerli 1999, Seattle
  beerli@genetics.washington.edu
- $Id: reporter.c,v 1.10 2000/04/04 18:39:44 beerli Exp $
+ $Id: reporter.c,v 1.13 2001/03/09 16:29:19 beerli Exp $
 
 -------------------------------------------------------*/
 
@@ -30,18 +30,13 @@
 #endif
 
 
-void both_chain_means (double *mc, double *lc, double *tc, long len, long lastn,
-		       long n);
-void calc_gelmanb (double *gelmanb, double *mc, double *tc, double *lc,
-		   long len, long lastn, long n);
-void calc_gelmanw (double *gelmanw, world_fmt * world, double *mc, double *tc,
-		   long len, long lastn, long n);
-void calc_gelmanr (double *gelmanr, double *gelmanw, double *gelmanb, long len,
-		   long lastn, long n);
-void calc_average_biggest_gelmanr (double *gelmanr, long len, double *meanR,
-				   double *bigR);
+void both_chain_means (double *mc, double *lc, double *tc, long len, long lastn, long n);
+void calc_gelmanb (double *gelmanb, double *mc, double *tc, double *lc, long len, long lastn, long n);
+void calc_gelmanw (double *gelmanw, world_fmt * world, double *mc, double *tc, long len, long lastn, long n);
+void calc_gelmanr (double *gelmanr, double *gelmanw, double *gelmanb, long len, long lastn, long n);
+void calc_average_biggest_gelmanr (double *gelmanr, long len, double *meanR, double *bigR);
 void print_gelmanr (double average, double biggest);
-double calc_s (long this, double *tc, world_fmt * world);
+double calc_s (long tthis, double *tc, world_fmt * world);
 void chain_means (double *thischainmeansm, world_fmt * world);
 
 
@@ -57,7 +52,7 @@ convergence_check (world_fmt * world, boolean progress)
   static long lastn = 0;
   static boolean first = TRUE;
   long n = 0;
-  if (world->chains < 2 || world->repkind!=SINGLECHAIN)
+  if (world->chains < 2 || world->repkind != SINGLECHAIN)
     return;
 
   if (world->start)
@@ -88,15 +83,11 @@ convergence_check (world_fmt * world, boolean progress)
       else
 	{
 	  chain_means (thischainmeans, world);
-	  both_chain_means (chainmeans, lastchainmeans, thischainmeans, len,
-			    lastn, n);
-	  calc_gelmanb (gelmanb, chainmeans, thischainmeans, lastchainmeans,
-			len, lastn, n);
-	  calc_gelmanw (gelmanw, world, thischainmeans, lastchainmeans, len,
-			lastn, n);
+	  both_chain_means (chainmeans, lastchainmeans, thischainmeans, len, lastn, n);
+	  calc_gelmanb (gelmanb, chainmeans, thischainmeans, lastchainmeans, len, lastn, n);
+	  calc_gelmanw (gelmanw, world, thischainmeans, lastchainmeans, len, lastn, n);
 	  calc_gelmanr (gelmanr, gelmanw, gelmanb, len, lastn, n);
-	  calc_average_biggest_gelmanr (gelmanr, len, &world->gelmanmeanR,
-					&world->gelmanmaxR);
+	  calc_average_biggest_gelmanr (gelmanr, len, &world->gelmanmeanR, &world->gelmanmaxR);
 	  memcpy (lastchainmeans, thischainmeans, sizeof (double) * len);
 	  lastn = n;
 	}
@@ -104,8 +95,7 @@ convergence_check (world_fmt * world, boolean progress)
 }
 
 void
-both_chain_means (double *mc, double *lc, double *tc, long len, long lastn,
-		  long n)
+both_chain_means (double *mc, double *lc, double *tc, long len, long lastn, long n)
 {
   long i;
 
@@ -117,23 +107,20 @@ both_chain_means (double *mc, double *lc, double *tc, long len, long lastn,
 
 
 void
-calc_gelmanb (double *gelmanb, double *mc, double *tc, double *lc, long len,
-	      long lastn, long n)
+calc_gelmanb (double *gelmanb, double *mc, double *tc, double *lc, long len, long lastn, long n)
 {
   long i;
   double nn = (n + lastn) / 2.;
 
   for (i = 0; i < len; i++)
     {
-      gelmanb[i] =
-	nn * (pow ((lc[i] - mc[i]), 2.) + (pow ((tc[i] - mc[i]), 2.)));
+      gelmanb[i] = nn * (pow ((lc[i] - mc[i]), 2.) + (pow ((tc[i] - mc[i]), 2.)));
     }
 
 }
 
 void
-calc_gelmanw (double *gelmanw, world_fmt * world, double *mc, double *tc,
-	      long len, long lastn, long n)
+calc_gelmanw (double *gelmanw, world_fmt * world, double *mc, double *tc, long len, long lastn, long n)
 {
   long i;
   double s1, s2;
@@ -149,23 +136,19 @@ calc_gelmanw (double *gelmanw, world_fmt * world, double *mc, double *tc,
 
 
 void
-calc_gelmanr (double *gelmanr, double *gelmanw, double *gelmanb, long len,
-	      long lastn, long n)
+calc_gelmanr (double *gelmanr, double *gelmanw, double *gelmanb, long len, long lastn, long n)
 {
   long i;
   double nn = (n + lastn) / 2.;
 
   for (i = 0; i < len; i++)
     {
-      gelmanr[i] =
-	sqrt (((nn - 1.) / nn * gelmanw[i] + 1. / nn * gelmanb[i]) /
-	      gelmanw[i]);
+      gelmanr[i] = sqrt (((nn - 1.) / nn * gelmanw[i] + 1. / nn * gelmanb[i]) / gelmanw[i]);
     }
 }
 
 void
-calc_average_biggest_gelmanr (double *gelmanr, long len, double *meanR,
-			      double *bigR)
+calc_average_biggest_gelmanr (double *gelmanr, long len, double *meanR, double *bigR)
 {
   long i;
   double average = 0;
@@ -176,7 +159,10 @@ calc_average_biggest_gelmanr (double *gelmanr, long len, double *meanR,
 	biggest = gelmanr[i];
       average += gelmanr[i];
     }
-  *meanR = average / len;
+  if(len>0)
+    *meanR = average / len;
+  else
+    *meanR = average; 
   *bigR = biggest;
 }
 
@@ -184,14 +170,12 @@ void
 print_gelmanr (double average, double biggest)
 {
   fprintf (stdout, "           Average Gelman's R = %f\n", average);
-  fprintf (stdout,
-	   "           Largest Gelman's R = %f [Value < 1.2 show convergence]\n",
-	   biggest);
+  fprintf (stdout, "           Largest Gelman's R = %f [Value < 1.2 show convergence]\n", biggest);
 }
 
 
 double
-calc_s (long this, double *tc, world_fmt * world)
+calc_s (long tthis, double *tc, world_fmt * world)
 {
   long i, j;
   double s = 0;
@@ -205,47 +189,39 @@ calc_s (long this, double *tc, world_fmt * world)
       startp = startkm + world->numpop;
       startl = startp + world->numpop;
     }
-  if (this < startkm)
+  if (tthis < startkm)
     {
-      i = this;
+      i = tthis;
       for (j = 0; j < world->atl[rep][world->locus].T; j++)
-	s +=
-	  (world->atl[rep][world->locus].tl[j].kt[i] - tc[i]) * (world->atl[rep][world->locus].tl[j].kt[i] -
-						 tc[i]);
+	s += (world->atl[rep][world->locus].tl[j].kt[i] - tc[i]) * (world->atl[rep][world->locus].tl[j].kt[i] - tc[i]);
       s /= world->atl[rep][world->locus].T - 1.;
       return s;
     }
   else
     {
-      if (this < startp)
+      if (tthis < startp)
 	{
-	  i = this - startkm;
+	  i = tthis - startkm;
 	  for (j = 0; j < world->atl[rep][world->locus].T; j++)
-	    s +=
-	      (world->atl[rep][world->locus].tl[j].km[i] - tc[i]) * (world->atl[rep][world->locus].tl[j].km[i] -
-						     tc[i]);
+	    s += (world->atl[rep][world->locus].tl[j].km[i] - tc[i]) * (world->atl[rep][world->locus].tl[j].km[i] - tc[i]);
 	  s /= world->atl[rep][world->locus].T - 1.;
 	  return s;
 	}
       else
 	{
-	  if (this < startl)
+	  if (tthis < startl)
 	    {
-	      i = this - startp;
+	      i = tthis - startp;
 	      for (j = 0; j < world->atl[rep][world->locus].T; j++)
-		s +=
-		  (world->atl[rep][world->locus].tl[j].p[i] -
-		   tc[i]) * (world->atl[rep][world->locus].tl[j].p[i] - tc[i]);
+		s += (world->atl[rep][world->locus].tl[j].p[i] - tc[i]) * (world->atl[rep][world->locus].tl[j].p[i] - tc[i]);
 	      s /= world->atl[rep][world->locus].T - 1.;
 	      return s;
 	    }
 	  else
 	    {
-	      i = this - startl;
+	      i = tthis - startl;
 	      for (j = 0; j < world->atl[rep][world->locus].T; j++)
-		s +=
-		  (world->atl[rep][world->locus].tl[j].mindex[i] -
-		   tc[i]) * (world->atl[rep][world->locus].tl[j].mindex[i] - tc[i]);
+		s += (world->atl[rep][world->locus].tl[j].mindex[i] - tc[i]) * (world->atl[rep][world->locus].tl[j].mindex[i] - tc[i]);
 	      s /= world->atl[rep][world->locus].T - 1.;
 	      return s;
 	    }
@@ -259,17 +235,17 @@ calc_s (long this, double *tc, world_fmt * world)
 void
 chain_means (double *thischainmeans, world_fmt * world)
 {
-  static long startp, startl, startkm;
-  static boolean done = FALSE;
+  /*static*/long startp, startl, startkm;
+
   long i, j;
 
-  if (!done)
-    {
-      done = TRUE;
+  //if (!done)
+  //  {
+  //    done = TRUE;
       startkm = world->numpop;
       startp = startkm + world->numpop;
       startl = startp + world->numpop;
-    }
+      // }
 
   for (j = 0; j < world->atl[world->rep][world->locus].T; j++)
     {
@@ -292,3 +268,8 @@ chain_means (double *thischainmeans, world_fmt * world)
     thischainmeans[i] /= world->atl[world->rep][world->locus].T;
 
 }
+
+
+
+
+
